@@ -31,11 +31,18 @@
  +-------------------------------------------------------------------------+
 */
 
-extern int db_insert(MYSQL *mysql, const char *query);
-extern MYSQL_RES *db_query(MYSQL *mysql, const char *query);
-extern void db_connect(const char *database, MYSQL *mysql);
+extern int db_insert(MYSQL *mysql, int type, const char *query);
+extern MYSQL_RES *db_query(MYSQL *mysql, int type, const char *query);
+extern void db_connect(int type, MYSQL *mysql);
 extern void db_disconnect(MYSQL *mysql);
-extern void db_escape(MYSQL *mysql, char *output, const char *input);
+extern void db_escape(MYSQL *mysql, char *output, int max_size, const char *input);
 extern void db_free_result(MYSQL_RES *result);
 
 extern int append_hostrange(char *obuf, const char *colname);
+
+#define MYSQL_SET_OPTION(opt, value, desc)	\
+	options_error = mysql_options(mysql, opt, value); \
+	if (options_error < 0) {\
+	        die("FATAL: MySQL options unable to set %s option", desc);\
+	}\
+
